#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "linear.h"
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL

static char *line;
static int max_line_len;

static char* readline(FILE *input)
{
	int len;
	
	if(fgets(line,max_line_len,input) == NULL)
		return NULL;

	while(strrchr(line,'\n') == NULL)
	{
		max_line_len *= 2;
		line = (char *) realloc(line,max_line_len);
		len = (int) strlen(line);
		if(fgets(line+len,max_line_len-len,input) == NULL)
			break;
	}
	return line;
}

void parse_command_line(int argc, char **argv, char *input_file_name, char *model_file_name);
void read_problem(const char *filename);
void do_cross_validation();

struct feature_node *x_space;
struct parameter param;
struct problem prob;
struct model* model_;
int flag_cross_validation;
int nr_fold;
double bias;

int main(int argc, char **argv)
{

	read_problem(argv[1]);

	free(prob.y);
	free(prob.x);
	free(x_space);

	return 0;
}

// read in a problem (in libsvm format)
void read_problem(const char *filename)
{
	int max_index = 0, index = 0;
	long elements;
	long start,  n_i;
	double w_s =1.0;
	double n_hat = 0, n_bar = 0;
	FILE *fp = fopen(filename,"r");
	char *endptr;
	double n;	

	if(fp == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",filename);
		exit(1);
	}

	prob.l = 0;
	elements = 0;
	max_line_len = 1024;

	line = Malloc(char,max_line_len);
	while(readline(fp)!=NULL)
	{
		char *p = strtok(line," \t"); // label

		while(1)
		{
			p = strtok(NULL," \t");
			if(p == NULL || *p == '\n')
				break;
		index = (int) strtol(p,&endptr, 10);		
		}		

		if(index>max_index)
			max_index = index;
	}		
	n = max_index;
	w_s = ((double)n+2)*((double)n+1)/2.0;
	rewind(fp);	
	while(readline(fp)!=NULL)
	{
		char *p = strtok(line," \t"); // label
		start = elements;
		// features
		while(1)
		{
			p = strtok(NULL," \t");
			if(p == NULL || *p == '\n')
				break;
			elements++;
		}
		n_i = elements - start;
		n_bar += (double)n_i;
		n_hat += ((double)n_i+2)*((double)n_i+1)/2;		
		w_s *= ( 1.0 - (double)n_i*((double)n_i -1.0)/n/(n-1.0));				
		prob.l++;
	}	
	n_bar = n_bar / prob.l;
	n_hat = n_hat / prob.l;
	printf("nbar %lf\n", n_bar);
	printf("nbar^2 %lf\n",n_bar*n_bar/2);
	printf("nhat %lf\n",n_hat);
	printf("l*nbar %lf\n",n_bar*prob.l);
	printf("n^2 %lf\n",(n+2)*(n+1)/2);
	printf("w %lf\n", (n+2)*(n+1)/2 -  w_s);	
	fclose(fp);
}
